-module(ai_optimizer).
-export([optimize_ai_system/2, tune_parameters/2]).

-include("../include/game_records.hrl").

%% 分析性能指标
analyze_performance_metrics(Metrics) ->
    % 分析AI性能指标
    #{
        win_rate => maps:get(win_rate, Metrics, 0.0),
        avg_decision_time => maps:get(avg_decision_time, Metrics, 0.0),
        strategy_effectiveness => maps:get(strategy_effectiveness, Metrics, 0.0),
        learning_rate => maps:get(learning_rate, Metrics, 0.01)
    }.

%% 优化决策系统
optimize_decision_system(AIState, PerformanceAnalysis) ->
    % 根据性能分析优化决策系统
    CurrentSystem = AIState#ai_state.strategy_model,
    WinRate = maps:get(win_rate, PerformanceAnalysis),
    
    % 根据胜率调整策略参数
    AdjustedParams = adjust_strategy_parameters(CurrentSystem, WinRate),
    
    % 返回优化后的决策系统
    CurrentSystem#{parameters => AdjustedParams}.

%% 优化学习系统
optimize_learning_system(AIState, PerformanceAnalysis) ->
    % 根据性能分析优化学习系统
    CurrentSystem = AIState#ai_state.learning_model,
    LearningRate = maps:get(learning_rate, PerformanceAnalysis),
    
    % 调整学习率
    AdjustedLearningRate = adjust_learning_rate(LearningRate, PerformanceAnalysis),
    
    % 返回优化后的学习系统
    CurrentSystem#{learning_rate => AdjustedLearningRate}.

optimize_ai_system(AIState, Metrics) ->
    % 分析性能指标
    PerformanceAnalysis = analyze_performance_metrics(Metrics),
    
    % 优化决策系统
    OptimizedDecisionSystem = optimize_decision_system(AIState, PerformanceAnalysis),
    
    % 优化学习系统
    OptimizedLearningSystem = optimize_learning_system(AIState, PerformanceAnalysis),
    
    % 更新AI状态
    AIState#ai_state{
        strategy_model = OptimizedDecisionSystem,
        learning_model = OptimizedLearningSystem
    }.

tune_parameters(Parameters, Performance) ->
    % 参数优化逻辑
    OptimizedParams = lists:map(
        fun({Param, Value}) ->
            NewValue = adjust_parameter(Param, Value, Performance),
            {Param, NewValue}
        end,
        Parameters
    ),
    maps:from_list(OptimizedParams).

%% 内部辅助函数

%% 调整策略参数
adjust_strategy_parameters(Strategy, WinRate) ->
    CurrentParams = maps:get(parameters, Strategy, #{}),
    
    % 根据胜率调整参数
    maps:map(
        fun(ParamName, ParamValue) ->
            adjust_parameter(ParamName, ParamValue, #{win_rate => WinRate})
        end,
        CurrentParams
    ).

%% 调整学习率
adjust_learning_rate(CurrentRate, PerformanceAnalysis) ->
    % 根据性能指标调整学习率
    Effectiveness = maps:get(strategy_effectiveness, PerformanceAnalysis, 0.5),
    
    % 如果效果好，减小学习率；如果效果差，增大学习率
    case Effectiveness of
        E when E > 0.7 -> max(0.001, CurrentRate * 0.9);
        E when E < 0.3 -> min(0.1, CurrentRate * 1.1);
        _ -> CurrentRate
    end.

%% 调整单个参数
adjust_parameter(Param, Value, Performance) ->
    % 根据性能调整参数值
    WinRate = maps:get(win_rate, Performance, 0.5),
    
    % 简单的参数调整逻辑
    case Param of
        risk_factor when WinRate < 0.4 ->
            % 胜率低时降低风险
            max(0.1, Value * 0.9);
        risk_factor when WinRate > 0.6 ->
            % 胜率高时可以增加风险
            min(0.9, Value * 1.1);
        aggressive_factor when WinRate < 0.4 ->
            % 胜率低时降低激进程度
            max(0.1, Value * 0.9);
        aggressive_factor when WinRate > 0.6 ->
            % 胜率高时可以更激进
            min(0.9, Value * 1.1);
        _ ->
            % 其他参数保持不变
            Value
    end.