-module(ai_player).
-behaviour(gen_server).

-export([start_link/1, init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).
-export([create_ai_player/1, play_turn/1]).

-record(state, {
    name,
    game_pid,
    cards = [],
    difficulty = normal,  % easy | normal | hard
    strategy = undefined, % landlord | farmer
    last_play = []
}).

%% AI难度设置
-define(EASY_THINK_TIME, 1000).    % 1秒
-define(NORMAL_THINK_TIME, 500).   % 0.5秒
-define(HARD_THINK_TIME, 200).     % 0.2秒

%% API
start_link(Name) ->
    gen_server:start_link(?MODULE, [Name], []).

create_ai_player(Difficulty) ->
    Name = generate_ai_name(),
    {ok, Pid} = start_link(Name),
    gen_server:cast(Pid, {set_difficulty, Difficulty}),
    {ok, Pid}.

play_turn(AiPid) ->
    gen_server:cast(AiPid, play_turn).

%% Callbacks
init([Name]) ->
    {ok, #state{name = Name}}.

handle_call(get_name, _From, State) ->
    {reply, {ok, State#state.name}, State};

handle_call(_Request, _From, State) ->
    {reply, {error, unknown_call}, State}.

handle_cast({set_difficulty, Difficulty}, State) ->
    {noreply, State#state{difficulty = Difficulty}};

handle_cast({update_cards, Cards}, State) ->
    {noreply, State#state{cards = Cards}};

handle_cast({set_strategy, Strategy}, State) ->
    {noreply, State#state{strategy = Strategy}};

handle_cast(play_turn, State) ->
    timer:sleep(get_think_time(State#state.difficulty)),
    {Play, NewState} = calculate_best_play(State),
    case Play of
        pass ->
            game_server:pass(State#state.game_pid, self());
        Cards ->
            game_server:play_cards(State#state.game_pid, self(), Cards)
    end,
    {noreply, NewState};

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_info(_Info, State) ->
    {noreply, State}.

%% 内部函数

% 生成AI玩家名称
generate_ai_name() ->
    Names = ["AlphaBot", "DeepPlayer", "SmartAI", "MasterBot", "ProBot"],
    RandomName = lists:nth(rand:uniform(length(Names)), Names),
    RandomNum = integer_to_list(rand:uniform(999)),
    RandomName ++ "_" ++ RandomNum.

% 获取思考时间
get_think_time(easy) -> ?EASY_THINK_TIME;
get_think_time(normal) -> ?NORMAL_THINK_TIME;
get_think_time(hard) -> ?HARD_THINK_TIME.

% 计算最佳出牌
calculate_best_play(State = #state{cards = Cards, strategy = Strategy, last_play = LastPlay}) ->
    case Strategy of
        landlord -> calculate_landlord_play(Cards, LastPlay, State);
        farmer -> calculate_farmer_play(Cards, LastPlay, State)
    end.

% 地主策略
calculate_landlord_play(Cards, LastPlay, State) ->
    case should_play_big(Cards, LastPlay, State) of
        true ->
            {get_biggest_play(Cards, LastPlay), State};
        false ->
            {get_optimal_play(Cards, LastPlay), State}
    end.

% 农民策略
calculate_farmer_play(Cards, LastPlay, State) ->
    case should_save_cards(Cards, LastPlay, State) of
        true ->
            {pass, State};
        false ->
            {get_optimal_play(Cards, LastPlay), State}
    end.

% 判断是否应该出大牌
should_play_big(Cards, LastPlay, _State) ->
    RemainingCount = length(Cards),
    case RemainingCount of
        1 -> true;
        2 -> true;
        _ -> 
            has_winning_chance(Cards, LastPlay)
    end.

% 判断是否应该保留手牌
should_save_cards(Cards, LastPlay, _State) ->
    case LastPlay of
        [] -> false;
        _ ->
            RemainingCount = length(Cards),
            OtherPlayerCards = estimate_other_players_cards(),
            RemainingCount < 5 andalso OtherPlayerCards > RemainingCount * 2
    end.

% 估算其他玩家的牌数
estimate_other_players_cards() ->
    15. % 这是一个简化的估算，实际实现需要根据游戏进程动态计算

% 检查是否有获胜机会
has_winning_chance(Cards, _LastPlay) ->
    % 简化版本的获胜概率计算
    length(Cards) =< 4.

% 获取最大的可出牌组合
get_biggest_play(Cards, LastPlay) ->
    % 实现查找最大牌型的逻辑
    % 这里需要调用 card_rules 模块来判断牌型
    case LastPlay of
        [] -> 
            find_biggest_combination(Cards);
        _ ->
            find_bigger_combination(Cards, LastPlay)
    end.

% 获取最优的出牌组合
get_optimal_play(Cards, LastPlay) ->
    % 实现最优出牌策略
    % 考虑因素：手牌数量、牌型组合、出牌节奏等
    case LastPlay of
        [] ->
            find_optimal_combination(Cards);
        _ ->
            find_minimum_bigger_combination(Cards, LastPlay)
    end.

% 查找最大牌型组合
find_biggest_combination(Cards) ->
    % 按照牌型优先级查找：火箭 > 炸弹 > 顺子 > 三带二 > 三带一 > 对子 > 单牌
    case find_rocket(Cards) of
        {ok, Rocket} -> Rocket;
        _ -> 
            case find_bomb(Cards) of
                {ok, Bomb} -> Bomb;
                _ -> find_best_normal_combination(Cards)
            end
    end.

% 查找比上家更大的牌型
find_bigger_combination(Cards, LastPlay) ->
    LastType = card_rules:get_card_type(LastPlay),
    case LastType of
        rocket -> []; % 火箭最大，无法压过
        bomb -> find_bigger_bomb(Cards, LastPlay);
        _ -> find_bigger_normal_combination(Cards, LastPlay, LastType)
    end.

% 查找最优组合
find_optimal_combination(Cards) ->
    % 简化实现：优先出单牌、对子、三带一等小牌型
    case find_single(Cards) of
        {ok, Single} -> Single;
        _ -> find_biggest_combination(Cards)
    end.

% 查找最小的能压过上家的牌
find_minimum_bigger_combination(Cards, LastPlay) ->
    % 简化实现：找到能压过上家的最小牌组合
    Bigger = find_bigger_combination(Cards, LastPlay),
    case Bigger of
        [] -> [];
        _ -> Bigger
    end.

% 处理代码热更新
code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

% 处理进程终止
terminate(_Reason, _State) ->
    ok.