-module(game_server).
-behaviour(gen_server).

-export([start_link/0, init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).
-export([create_game/0, join_game/2, start_game/1, play_cards/3, pass/2]).

-record(state, {
    players = [],         % [{PlayerPid, Cards}]
    current_player = none,
    last_play = [],
    landlord = none,
    game_status = waiting % waiting | playing | finished
}).

%% API
start_link() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

create_game() ->
    gen_server:call(?MODULE, create_game).

join_game(GamePid, PlayerPid) ->
    gen_server:call(GamePid, {join_game, PlayerPid}).

start_game(GamePid) ->
    gen_server:call(GamePid, start_game).

play_cards(GamePid, PlayerPid, Cards) ->
    gen_server:call(GamePid, {play_cards, PlayerPid, Cards}).

pass(GamePid, PlayerPid) ->
    gen_server:call(GamePid, {pass, PlayerPid}).

%% Callbacks
init([]) ->
    {ok, #state{}}.

handle_call(create_game, _From, State) ->
    {reply, {ok, self()}, State#state{players = [], game_status = waiting}};

handle_call({join_game, PlayerPid}, _From, State = #state{players = Players}) ->
    case length(Players) < 3 of
        true ->
            NewPlayers = [{PlayerPid, []} | Players],
            {reply, {ok, length(NewPlayers)}, State#state{players = NewPlayers}};
        false ->
            {reply, {error, game_full}, State}
    end;

handle_call(start_game, _From, State = #state{players = Players}) ->
    case length(Players) =:= 3 of
        true ->
            % 初始化并洗牌
            Cards = cards:shuffle_cards(cards:init_cards()),
            {P1Cards, P2Cards, P3Cards, LandlordCards} = cards:deal_cards(Cards),
            % 随机选择地主
            LandlordIndex = rand:uniform(3),
            {NewPlayers, NewLandlord} = assign_cards_and_landlord(Players, [P1Cards, P2Cards, P3Cards], LandlordCards, LandlordIndex),
            {reply, {ok, NewLandlord}, State#state{
                players = NewPlayers,
                current_player = NewLandlord,
                game_status = playing,
                landlord = NewLandlord
            }};
        false ->
            {reply, {error, not_enough_players}, State}
    end;

handle_call({play_cards, PlayerPid, Cards}, _From, State = #state{current_player = PlayerPid, last_play = LastPlay, players = Players}) ->
    case card_rules:validate_play(Cards, LastPlay) of
        true ->
            case remove_cards(PlayerPid, Cards, Players) of
                {ok, NewPlayers} ->
                    NextPlayer = get_next_player(PlayerPid, Players),
                    case check_winner(NewPlayers) of
                        {winner, Winner} ->
                            {reply, {ok, winner, Winner}, State#state{
                                players = NewPlayers,
                                game_status = finished
                            }};
                        no_winner ->
                            {reply, {ok, next_player, NextPlayer}, State#state{
                                players = NewPlayers,
                                current_player = NextPlayer,
                                last_play = Cards
                            }}
                    end;
                error ->
                    {reply, {error, invalid_cards}, State}
            end;
        false ->
            {reply, {error, invalid_play}, State}
    end;

handle_call({pass, PlayerPid}, _From, State = #state{current_player = PlayerPid, last_play = LastPlay, players = Players}) ->
    case LastPlay of
        [] ->
            {reply, {error, cannot_pass}, State};
        _ ->
            NextPlayer = get_next_player(PlayerPid, Players),
            {reply, {ok, next_player, NextPlayer}, State#state{current_player = NextPlayer}}
    end;

handle_call(_, _From, State) ->
    {reply, {error, invalid_call}, State}.

handle_cast(_, State) ->
    {noreply, State}.

handle_info(_, State) ->
    {noreply, State}.

terminate(_Reason, _State) ->
    ok.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% 内部辅助函数

%% 分配牌和地主
assign_cards_and_landlord(Players, [P1, P2, P3], LandlordCards, LandlordIndex) ->
    {Pid1, _} = lists:nth(1, Players),
    {Pid2, _} = lists:nth(2, Players),
    {Pid3, _} = lists:nth(3, Players),
    
    case LandlordIndex of
        1 ->
            {[{Pid1, P1 ++ LandlordCards}, {Pid2, P2}, {Pid3, P3}], Pid1};
        2 ->
            {[{Pid1, P1}, {Pid2, P2 ++ LandlordCards}, {Pid3, P3}], Pid2};
        3 ->
            {[{Pid1, P1}, {Pid2, P2}, {Pid3, P3 ++ LandlordCards}], Pid3}
    end.

%% 获取下一个玩家
get_next_player(CurrentPid, Players) ->
    PlayerPids = [Pid || {Pid, _} <- Players],
    CurrentIndex = get_player_index(CurrentPid, PlayerPids),
    lists:nth(1 + ((CurrentIndex) rem 3), PlayerPids).

%% 获取玩家索引
get_player_index(Pid, Pids) ->
    {Index, _} = lists:foldl(
        fun(P, {Idx, Found}) ->
            case P =:= Pid of
                true -> {Idx, Idx};
                false -> {Idx + 1, Found}
            end
        end,
        {1, none},
        Pids
    ),
    Index.

%% 移除玩家手中的牌
remove_cards(PlayerPid, CardsToRemove, Players) ->
    case lists:keyfind(PlayerPid, 1, Players) of
        {PlayerPid, PlayerCards} ->
            case can_remove_cards(CardsToRemove, PlayerCards) of
                true ->
                    NewCards = PlayerCards -- CardsToRemove,
                    NewPlayers = lists:keyreplace(PlayerPid, 1, Players, {PlayerPid, NewCards}),
                    {ok, NewPlayers};
                false ->
                    error
            end;
        false ->
            error
    end.

%% 检查是否能移除指定的牌
can_remove_cards(CardsToRemove, PlayerCards) ->
    lists:all(
        fun(Card) ->
            lists:member(Card, PlayerCards)
        end,
        CardsToRemove
    ).

%% 检查是否有获胜者
check_winner(Players) ->
    case lists:filter(
        fun({_, Cards}) ->
            length(Cards) =:= 0
        end,
        Players
    ) of
        [{Winner, _}|_] -> {winner, Winner};
        [] -> no_winner
    end.