-module(optimizer).
-export([update_adam/3, update_sgd/3, init_adam_state/0]).

-record(adam_state, {
    m = #{},     % First moment
    v = #{},     % Second moment
    t = 0,       % Timestamp
    beta1 = 0.9, % Exponential decay rate for first moment
    beta2 = 0.999, % Exponential decay rate for second moment
    epsilon = 1.0e-8
}).

init_adam_state() ->
    #adam_state{}.

update_adam(Params, Gradients, State = #adam_state{t = T}) ->
    NewT = T + 1,
    {NewParams, NewM, NewV} = maps:fold(
        fun(Key, Grad, {ParamsAcc, MAcc, VAcc}) ->
            M = maps:get(Key, State#adam_state.m, 0.0),
            V = maps:get(Key, State#adam_state.v, 0.0),
            
            % Update biased first moment estimate
            NewM1 = State#adam_state.beta1 * M + (1 - State#adam_state.beta1) * Grad,
            % Update biased second moment estimate
            NewV1 = State#adam_state.beta2 * V + (1 - State#adam_state.beta2) * Grad * Grad,
            
            % Compute bias-corrected first moment estimate
            MHat = NewM1 / (1 - math:pow(State#adam_state.beta1, NewT)),
            % Compute bias-corrected second moment estimate
            VHat = NewV1 / (1 - math:pow(State#adam_state.beta2, NewT)),
            
            % Update parameters
            Param = maps:get(Key, Params),
            NewParam = Param - State#adam_state.epsilon * MHat / (math:sqrt(VHat) + State#adam_state.epsilon),
            
            {
                maps:put(Key, NewParam, ParamsAcc),
                maps:put(Key, NewM1, MAcc),
                maps:put(Key, NewV1, VAcc)
            }
        end,
        {Params, State#adam_state.m, State#adam_state.v},
        Gradients
    ),
    {NewParams, State#adam_state{m = NewM, v = NewV, t = NewT}}.

update_sgd(Params, Gradients, LearningRate) ->
    maps:map(
        fun(Key, Param) ->
            Grad = maps:get(Key, Gradients),
            Param - LearningRate * Grad
        end,
        Params
    ).